import json
import platform

from requests import Session
from requests.adapters import HTTPAdapter
from requests.packages.urllib3.util.retry import Retry

import magic_admin
from magic_admin import version
from magic_admin.config import api_secret_api_key_missing_message
from magic_admin.config import base_url
from magic_admin.error import APIConnectionError
from magic_admin.error import APIError
from magic_admin.error import AuthenticationError
from magic_admin.error import BadRequestError
from magic_admin.error import ForbiddenError
from magic_admin.error import RateLimitingError
from magic_admin.response import MagicResponse


class RequestsClient:

    def __init__(self, retries, timeout, backoff_factor):
        self._retries = retries
        self._timeout = timeout
        self._backoff_factor = backoff_factor

        self._setup_request_session()

    @staticmethod
    def _get_platform_info():
        platform_info = {}

        for attr, func in [
            ['platform', platform.platform],
            ['language_version', platform.python_version],
            ['uname', platform.uname],
        ]:
            try:
                val = str(func())
            except Exception as e:
                val = '<{}>'.format(str(e))

            platform_info[attr] = val

        return platform_info

    def _setup_request_session(self):
        """Take advantage of the ``requets.Session``. If client is making several
        requests to the same host, the underlying TCP connection will be reused,
        which can result in a significant performance increase.
        """
        self.http = Session()
        self.http.mount(
            base_url,
            HTTPAdapter(
                max_retries=Retry(
                    total=self._retries,
                    backoff_factor=self._backoff_factor,
                ),
            ),
        )

    def _get_request_headers(self):
        user_agent = {
            'language': 'python',
            'sdk_version': version.VERSION,
            'publisher': 'magic',
            'http_lib': self.__class__.__name__,
            **self._get_platform_info(),
        }

        if magic_admin.api_secret_key is None:
            raise AuthenticationError(api_secret_api_key_missing_message)

        return {
            'X-Magic-Secret-Key': magic_admin.api_secret_key,
            'User-Agent': json.dumps(user_agent),
        }

    def request(self, method, url, params=None, data=None):
        try:
            api_resp = self.http.request(
                method,
                url,
                params=params,
                # Requests auto-converts this to JSON and add content-type
                # `application/json`.
                json=data,
                headers=self._get_request_headers(),
                timeout=self._timeout,
            )
        except Exception as e:
            return self._handle_request_error(e)

        return self._parse_and_convert_to_api_response(
            api_resp,
            params,
            data,
        )

    def _parse_and_convert_to_api_response(self, resp, request_params, request_data):
        status_code = resp.status_code

        if 200 <= status_code < 300:
            return MagicResponse(resp.content, resp.json(), status_code)

        if status_code == 429:
            error_class = RateLimitingError
        elif status_code == 400:
            error_class = BadRequestError
        elif status_code == 401:
            error_class = AuthenticationError
        elif status_code == 403:
            error_class = ForbiddenError
        else:
            error_class = APIError

        resp_data = resp.json()
        raise error_class(
            http_status=resp_data.get('status'),
            http_code=status_code,
            http_resp_data=resp_data.get('data'),
            http_message=resp_data.get('message'),
            http_error_code=resp_data.get('error_code'),
            http_request_params=request_params,
            http_request_data=request_data,
            http_method=resp.request.method,
        )

    def _handle_request_error(self, e):
        message = 'Unexpected error thrown while communicating to Magic. ' \
            'Please reach out to support@magic.link if the problem continues. ' \
            'Error message: {error_class} was raised - {error_message}'.format(
                error_class=e.__class__.__name__,
                error_message=str(e) or 'no error message.',
            )

        raise APIConnectionError(message)



from magic_admin.error import DIDTokenError


def parse_public_address_from_issuer(issuer):
    """
    Args:
        issuer (str): Issuer (the signer, the "user"). This field is represented
            as a Decentralized Identifier populated with the user's Ethereum
            public key.

    Returns:
        public_address (str): An Ethereum public key.
    """
    try:
        return issuer.split(':')[2]
    except IndexError:
        raise DIDTokenError(
            'Given issuer ({}) is malformed. Please make sure it follows the '
            '`did:method-name:method-specific-id` format.'.format(issuer),
        )


def construct_issuer_with_public_address(public_address):
    return 'did:ethr:{}'.format(public_address)



from django.contrib.auth import get_user_model
from django.contrib.auth import login
from django.contrib.auth import logout
from django.contrib.auth.models import AnonymousUser
from django.utils.deprecation import MiddlewareMixin

from magic_admin.django.config import (
    MAGIC_AUTH_BACKEND,
    MAGIC_IDENTITY_KEY,
)
from magic_admin.django.exceptions import UnableToLoadUserFromIdentityToken
from magic_admin.error import (
    DIDTokenError,
    APIConnectionError,
    RateLimitingError,
    BadRequestError,
    AuthenticationError,
    ForbiddenError,
    APIError,
)
from magic_admin.utils.http import get_identity_token_from_header
from magic_admin.magic import Magic


user_model = get_user_model()


class MagicAuthMiddleware(MiddlewareMixin):

    @staticmethod
    def _persist_data_in_session(request, identity_token):
        request.session[MAGIC_IDENTITY_KEY] = identity_token
        request.session.modified = True

    @staticmethod
    def _load_identity_token_from_session(request):
        return request.session.get(MAGIC_IDENTITY_KEY, None)

    @staticmethod
    def _load_identity_token_from_header(request):
        return get_identity_token_from_header(request)

    @staticmethod
    def _is_request_related_to_magic(identity_token):
        return identity_token is not None

    @staticmethod
    def _try_loading_user_from_identity_token(identity_token):
        public_address = Magic().User.get_public_address(identity_token)

        try:
            return user_model.get_by_public_address(public_address)
        except user_model.DoesNotExist:
            return AnonymousUser()

    @staticmethod
    def can_user_be_logged_in(user):
        if user is None:
            return False

        is_active = getattr(user, 'is_active', None)
        return is_active or is_active is None

    def _attempt_handling_anonymous_user(self, request, identity_token):
        user = self._try_loading_user_from_identity_token(identity_token)

        if not user.is_anonymous and self.can_user_be_logged_in(user):
            # Log the user in to rehydrate the session.
            login(request, user, backend=MAGIC_AUTH_BACKEND)
        else:
            raise UnableToLoadUserFromIdentityToken()

    def _load_identity_token_from_sources(self, request):
        # The identity token from header take precedence over the one in session.
        return self._load_identity_token_from_header(
            request,
        ) or self._load_identity_token_from_session(request)

    def process_request(self, request):
        assert hasattr(request, 'user'), (
            'The Magic authentication middleware requires authentication '
            'middleware to be installed. Edit your MIDDLEWARE setting to insert '
            '`django.contrib.auth.middleware.AuthenticationMiddleware` before '
            '`magic_admin.django.auth.middleware.MagicAuthMiddleware`.'
        )

        identity_token = self._load_identity_token_from_sources(request)

        if not self._is_request_related_to_magic(identity_token):
            return

        try:
            Magic().Token.validate(identity_token)
        except (
                DIDTokenError,
                APIConnectionError,
                RateLimitingError,
                BadRequestError,
                AuthenticationError,
                ForbiddenError,
                APIError,
        ) as e:
            logout(request)
            raise e
            return

        if request.user.is_anonymous:
            try:
                self._attempt_handling_anonymous_user(request, identity_token)
            except UnableToLoadUserFromIdentityToken:
                return

        if (
            request.user.public_address and
            request.user.public_address != Magic().User.get_public_address(
                identity_token,
            )
        ):
            logout(request)
            return

        if self.can_user_be_logged_in(request.user):
            self._persist_data_in_session(request, identity_token)



from magic_admin.config import base_url
from magic_admin.http_client import RequestsClient


class ResourceMeta(type):

    def __init__(cls, name, bases, cls_dict):
        if not hasattr(cls, '_registry'):
            cls._registry = {}
        else:
            cls._registry[name] = cls()

        super().__init__(name, bases, cls_dict)


class ResourceComponent(metaclass=ResourceMeta):

    _base_url = base_url

    def __getattr__(self, resource_name):
        if resource_name in self._registry:
            return self._registry[resource_name]
        else:
            raise AttributeError(
                '{object_name} has no attribute \'{resource_name}\''.format(
                    object_name=self.__class__.__name__,
                    resource_name=resource_name,
                ),
            )

    def setup_request_client(self, retries, timeout, backoff_factor):
        _request_client = RequestsClient(retries, timeout, backoff_factor)

        for resource in self._registry.values():
            setattr(resource, '_request_client', _request_client)

    def _construct_url(self, url_path):
        return '{base_url}{url_path}'.format(
            base_url=self._base_url,
            url_path=url_path,
        )

    def request(self, method, url_path, params=None, data=None):
        return self._request_client.request(
            method.lower(),
            self._construct_url(url_path),
            params=params,
            data=data,
        )



from django.contrib.auth import get_user_model
from django.contrib.auth.backends import ModelBackend

from magic_admin.django.config import MagicAuthBackendMode
from magic_admin.django.exceptions import (
    MissingAuthorizationHeader,
    MissingUserEmailInput,
    PublicAddressDoesNotExist,
    UnsupportedAuthMode,
    UserEmailMissmatch,
)
from magic_admin.error import (
    DIDTokenError,
    APIConnectionError,
    RateLimitingError,
    BadRequestError,
    AuthenticationError,
    ForbiddenError,
    APIError,
)

from magic_admin.utils.http import get_identity_token_from_header
from magic_admin.magic import Magic
from magic_admin.utils.logging import (
    log_debug,
    log_info,
)


user_model = get_user_model()


class MagicAuthBackend(ModelBackend):

    @staticmethod
    def _load_user_from_email(email):
        log_debug('Loading user by email.', email=email)
        try:
            return user_model.objects.get(email=email)
        except user_model.DoesNotExist:
            return None

    @staticmethod
    def _validate_identity_token_and_load_user(
        identity_token,
        email,
        public_address,
    ):
        try:
            Magic().Token.validate(identity_token)
        except (
                DIDTokenError,
                APIConnectionError,
                RateLimitingError,
                BadRequestError,
                AuthenticationError,
                ForbiddenError,
                APIError,
        ) as e:
            log_debug(
                'DID Token failed validation. No user is to be retrieved.',
                error_class=e.__class__.__name__,
            )
            raise e
            return None

        try:
            user = user_model.get_by_public_address(public_address)
        except user_model.DoesNotExist:
            raise PublicAddressDoesNotExist()

        if user.email != email:
            raise UserEmailMissmatch()

        return user

    def user_can_authenticate(self, user):
        if user is None:
            return False

        return super().user_can_authenticate(user)

    def _update_user_with_public_address(self, user, public_address):
        if self.user_can_authenticate(user):
            user.update_user_with_public_address(
                user_id=None,
                public_address=public_address,
                user_obj=user,
            )

    def _handle_phantom_auth(self, request, email):
        identity_token = get_identity_token_from_header(request)
        if identity_token is None:
            raise MissingAuthorizationHeader()

        public_address = Magic().Token.get_public_address(identity_token)

        try:
            user = self._validate_identity_token_and_load_user(
                identity_token,
                email,
                public_address,
            )
        except PublicAddressDoesNotExist:
            user = self._load_user_from_email(email)
            if user is None:
                log_debug(
                    'User is not authenticated. No user found with the given email.',
                    email=email,
                )
                Magic().User.logout_by_public_address(public_address)
                return

            self._update_user_with_public_address(user, public_address)
        except UserEmailMissmatch as e:
            log_debug(
                'User is not authenticated. User email does not match for the '
                'public address.',
                email=email,
                public_address=public_address,
                error_class=e.__class__.__name__,
            )
            Magic().User.logout_by_public_address(public_address)
            return

        if self.user_can_authenticate(user):
            log_info('User authenticated with DID Token.')
            return user

    def authenticate(
        self,
        request,
        user_email=None,
        mode=MagicAuthBackendMode.MAGIC,
    ):
        if not user_email:
            raise MissingUserEmailInput()

        user_email = user_model.objects.normalize_email(user_email)

        if mode == MagicAuthBackendMode.MAGIC:
            return self._handle_phantom_auth(request, user_email)
        else:
            raise UnsupportedAuthMode()



import logging
import sys


# from fortmatic.config import LOGGER_NAME


LOGGER_NAME = 'magic'

LOG_LEVEl = 'debug'

logger = logging.getLogger(LOGGER_NAME)


def _magic_log_level():
    if LOG_LEVEl == 'debug':
        return LOG_LEVEl


def format_log(message, **kwargs):
    return dict(
        {'message': message},
        log_level=LOG_LEVEl,
        serverice=LOGGER_NAME,
        **kwargs,
    )


def log_debug(message, **kwargs):
    log_line = format_log(message, **kwargs)

    if _magic_log_level() == 'debug':
        print(log_line, file=sys.stderr)

    logger.debug(log_line)


def log_info(message, **kwargs):
    log_line = format_log(message, **kwargs)

    if _magic_log_level() == 'debug':
        print(log_line, file=sys.stderr)

    logger.info(log_line)



MAGIC_IDENTITY_KEY = '_magic_identity_token'
MAGIC_AUTH_BACKEND = 'magic_admin.django.auth.backends.MagicAuthBackend'


class MagicAuthBackendMode:

    DJANGO_DEFAULT_AUTH = 0
    MAGIC = 1

    ALLOWED_MODES = frozenset([DJANGO_DEFAULT_AUTH, MAGIC])



from magic_admin.resources.base import ResourceComponent
from magic_admin.utils.did_token import construct_issuer_with_public_address


class User(ResourceComponent):

    v1_user_info = '/v1/admin/auth/user/get'
    v2_user_logout = '/v2/admin/auth/user/logout'

    def get_metadata_by_issuer(self, issuer):
        return self.request('get', self.v1_user_info, params={'issuer': issuer})

    def get_metadata_by_public_address(self, public_address):
        return self.get_metadata_by_issuer(
            construct_issuer_with_public_address(public_address),
        )

    def get_metadata_by_token(self, did_token):
        return self.get_metadata_by_issuer(self.Token.get_issuer(did_token))

    def logout_by_issuer(self, issuer):
        return self.request('post', self.v2_user_logout, data={'issuer': issuer})

    def logout_by_public_address(self, public_address):
        return self.logout_by_issuer(
            construct_issuer_with_public_address(public_address),
        )

    def logout_by_token(self, did_token):
        return self.logout_by_issuer(self.Token.get_issuer(did_token))



import base64
import json

from eth_account.messages import defunct_hash_message
from web3.auto import w3

from magic_admin.error import DIDTokenError
from magic_admin.resources.base import ResourceComponent
from magic_admin.utils.did_token import parse_public_address_from_issuer
from magic_admin.utils.time import apply_did_token_nbf_grace_period
from magic_admin.utils.time import epoch_time_now


EXPECTED_DID_TOKEN_CONTENT_LENGTH = 2


class Token(ResourceComponent):

    required_fields = frozenset([
        'iat',
        'ext',
        'nbf',
        'iss',
        'sub',
        'aud',
        'tid',
    ])

    @classmethod
    def _check_required_fields(cls, claim):
        """
        Args:
            claim (dict): A dict that represents the claim portion of the DID
                token.

        Returns:
            None.
        """
        missing_fields = []
        for field in cls.required_fields:
            if field not in claim:
                missing_fields.append(field)

        if missing_fields:
            raise DIDTokenError(
                message='DID token is missing required field(s): {}'.format(
                    sorted(missing_fields),
                ),
            )

    @classmethod
    def decode(cls, did_token):
        """
        Args:
            did_token (base64.str): Base64 encoded string.

        Raises:
            DIDTokenError: If token format is invalid.

        Returns:
            proof (str): A signed message.
            claim (dict): A dict of unsigned message.
        """
        try:
            decoded_did_token = json.loads(
                base64.urlsafe_b64decode(did_token).decode('utf-8'),
            )
        except Exception as e:
            raise DIDTokenError(
                message='DID token is malformed. It has to be a based64 encoded '
                'JSON serialized string. {err} ({msg}).'.format(
                    err=e.__class__.__name__,
                    msg=str(e) or '<empty message>',
                ),
            )

        if len(decoded_did_token) != EXPECTED_DID_TOKEN_CONTENT_LENGTH:
            raise DIDTokenError(
                message='DID token is malformed. It has to have two parts '
                '[proof, claim].',
            )

        proof = decoded_did_token[0]

        try:
            claim = json.loads(decoded_did_token[1])
        except Exception as e:
            raise DIDTokenError(
                message='DID token is malformed. Given claim should be a JSON '
                'serialized string. {err} ({msg}).'.format(
                    err=e.__class__.__name__,
                    msg=str(e) or '<empty message>',
                ),
            )

        cls._check_required_fields(claim)

        return proof, claim

    @classmethod
    def get_issuer(cls, did_token):
        """
        Args:
            did_token (base64.str): Base64 encoded string.

        Returns:
            issuer (str): Issuer (the signer, the "user"). This field is represented
                as a Decentralized Identifier populated with the user's Ethereum
                public key.
        """
        _, claim = cls.decode(did_token)

        return claim['iss']

    @classmethod
    def get_public_address(cls, did_token):
        """
        Args:
            did_token (base64.str): Base64 encoded string.

        Returns:
            public_address (str): An Ethereum public key.
        """
        return parse_public_address_from_issuer(cls.get_issuer(did_token))

    @classmethod
    def validate(cls, did_token):
        """
        Args:
            did_token (base64.str): Base64 encoded string.

        Raises:
            DIDTokenError: If DID token fails the validation.

        Returns:
            None.
        """
        proof, claim = cls.decode(did_token)
        recovered_address = w3.eth.account.recoverHash(
            defunct_hash_message(
                text=json.dumps(claim, separators=(',', ':')),
            ),
            signature=proof,
        )

        if recovered_address != cls.get_public_address(did_token):
            raise DIDTokenError(
                message='Signature mismatch between "proof" and "claim". Please '
                'generate a new token with an intended issuer.',
            )

        current_time_in_s = epoch_time_now()

        if current_time_in_s > claim['ext']:
            raise DIDTokenError(
                message='Given DID token has expired. Please generate a new one.',
            )

        if current_time_in_s < apply_did_token_nbf_grace_period(claim['nbf']):
            raise DIDTokenError(
                message='Given DID token cannot be used at this time. Please '
                'check the "nbf" field and regenerate a new token with a suitable '
                'value.',
            )



class MagicError(Exception):

    def __init__(self, message=None):
        super().__init__(message)
        self._message = message

    def __str__(self):
        return self._message or '<empty message>'

    def __repr__(self):
        return '{error_class}(message={message!r})'.format(
            error_class=self.__class__.__name__,
            message=self._message,
        )

    def to_dict(self):
        return {'message': str(self)}


class DIDTokenError(MagicError):
    pass


class APIConnectionError(MagicError):
    pass


class RequestError(MagicError):

    def __init__(
        self,
        message=None,
        http_status=None,
        http_code=None,
        http_resp_data=None,
        http_message=None,
        http_error_code=None,
        http_request_params=None,
        http_request_data=None,
        http_method=None,
    ):
        super().__init__(message)
        self.http_status = http_status
        self.http_code = http_code
        self.http_resp_data = http_resp_data
        self.http_message = http_message
        self.http_error_code = http_error_code
        self.http_request_params = http_request_params
        self.http_request_data = http_request_data
        self.http_method = http_method

    def __repr__(self):
        return '{error_class}(message={message!r}, ' \
            'http_error_code={http_error_code}, ' \
            'http_code={http_code}).'.format(
                error_class=self.__class__.__name__,
                message=self._message or None,
                http_error_code=self.http_error_code or None,
                http_code=self.http_code or None,
            )

    def to_dict(self):
        _dict = super().to_dict()
        for attr in self.__dict__:
            if attr.startswith('http_'):
                _dict[attr] = self.__dict__[attr]

        return _dict


class RateLimitingError(RequestError):
    pass


class BadRequestError(RequestError):
    pass


class AuthenticationError(RequestError):
    pass


class ForbiddenError(RequestError):
    pass


class APIError(RequestError):
    pass



from django.contrib.auth import logout as django_logout

from magic_admin.magic import Magic
from magic_admin.utils.logging import log_debug


def logout(request):
    user = request.user

    if not user.is_anonymous and user.public_address:
        Magic().User.logout_by_public_address(user.public_address)
        log_debug('Log out user from Magic')

    django_logout(request)



from django.contrib.auth.models import AnonymousUser
from django.db import models
from django.utils.translation import gettext_lazy as _


class MagicUserMixin(models.Model):

    public_address = models.CharField(
        _('public address'),
        max_length=128,
        unique=True,
        blank=True,
        null=True,
        db_index=True,
        default=None,
    )

    class Meta:
        abstract = True

    @classmethod
    def get_by_public_address(cls, public_address):
        return cls.objects.get(public_address=public_address)

    def update_user_with_public_address(
        self,
        user_id,
        public_address,
        user_obj=None,
    ):
        if user_obj is None:
            user_obj = self.objects.get(pk=user_id)

        if user_obj.public_address == public_address:
            return user_obj

        user_obj.public_address = public_address
        user_obj.save(update_fields=['public_address'])

        return user_obj


class MagicAnonymousUser(AnonymousUser):
    pass



from os.path import dirname
from os.path import join

from setuptools import find_packages
from setuptools import setup

with open('README.md') as fh:
    long_description = fh.read()


def read_version():
    version_contents = {}
    with open(join(dirname(__file__), 'magic_admin', 'version.py')) as fh:
        exec(fh.read(), version_contents)

    return version_contents['VERSION']


def load_readme():
    with open(join(dirname(__file__), 'README.md')) as fh:
        long_description = fh.read()

    return long_description


def load_requirements():
    with open(join(dirname(__file__), 'requirements.txt')) as fh:
        requirements = fh.readlines()

    return requirements


setup(
    name='django-magic-fortmatic',
    version=read_version(),
    description='Magic Python Library',
    long_description=load_readme(),
    long_description_content_type='text/markdown',
    author='Magic',
    author_email='support@magic.link',
    url='https://magic.link',
    license='MIT',
    keywords='magic python sdk',
    packages=find_packages(
        exclude=[
            'tests',
            'tests.*',
            'testing',
            'testing.*',
            'virtualenv_run',
            'virtualenv_run.*',
        ],
    ),
    zip_safe=False,
    install_requires=load_requirements(),
    python_requires='>=3.6',
    project_urls={
        'Website': 'https://magic.link',
    },
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
)



class MagicDjangoException(Exception):
    pass


class UnsupportedAuthMode(MagicDjangoException):
    pass


class PublicAddressDoesNotExist(MagicDjangoException):
    pass


class UserEmailMissmatch(MagicDjangoException):
    pass


class MissingUserEmailInput(MagicDjangoException):
    pass


class MissingAuthorizationHeader(MagicDjangoException):
    pass


class UnableToLoadUserFromIdentityToken(MagicDjangoException):
    pass



import os

import magic_admin
from magic_admin.config import api_secret_api_key_missing_message
from magic_admin.error import AuthenticationError
from magic_admin.resources.base import ResourceComponent


RETRIES = 3
TIMEOUT = 10
BACKOFF_FACTOR = 0.02


class Magic:

    def __getattr__(self, attribute_name):
        try:
            return getattr(self._resource, attribute_name)
        except AttributeError:
            pass

        return super().__getattribute__(attribute_name)

    def __init__(
        self,
        api_secret_key=None,
        retries=RETRIES,
        timeout=TIMEOUT,
        backoff_factor=BACKOFF_FACTOR,
    ):
        self._resource = ResourceComponent()

        self._resource.setup_request_client(retries, timeout, backoff_factor)
        self._set_api_secret_key(api_secret_key)

    def _set_api_secret_key(self, api_secret_key):
        magic_admin.api_secret_key = api_secret_key or os.environ.get(
            'MAGIC_API_SECRET_KEY',
        )

        if magic_admin.api_secret_key is None:
            raise AuthenticationError(api_secret_api_key_missing_message)



import re

AUTHORIZATION_PATTERN = r'Bearer (?P<token>.+)'


def null_safe(value):
    if value is None or value in ['null', 'none', 'None', '']:
        return None

    return value


def parse_authorization_header_value(header_value):
    m = re.match(AUTHORIZATION_PATTERN, header_value)

    if m is None:
        return None

    return null_safe(m.group('token'))


def get_identity_token_from_header(request):
    header_value = request.META.get('HTTP_AUTHORIZATION', '')
    return parse_authorization_header_value(header_value)



base_url = 'https://api.magic.link'

api_secret_api_key_missing_message = 'API secret key is missing. Please specify ' \
    'an API secret key when you instantiate the `Magic(api_secret_key=<KEY>)` ' \
    'object or use the environment variable, `MAGIC_API_SECRET_KEY`. You can ' \
    'get your API secret key from https://dashboard.magic.link. If you are having ' \
    'trouble, please don\'t hesitate to reach out to us at support@magic.link'
